import { __read, __assign } from 'tslib';
import { defer, Subject, BehaviorSubject, combineLatest } from 'rxjs';
import { finalize, switchMap, startWith, shareReplay, map } from 'rxjs/operators';

function indicate(indicator) {
    return function (source) { return source.pipe(function (s) { return defer(function () {
        indicator.next(true);
        return s;
    }); }, finalize(function () { return indicator.next(false); })); };
}

var PaginationDataSource = /** @class */ (function () {
    function PaginationDataSource(endpoint, initialSort, initialQuery, pageSize) {
        var _this = this;
        if (pageSize === void 0) { pageSize = 20; }
        this.endpoint = endpoint;
        this.pageSize = pageSize;
        this.pageNumber = new Subject();
        this.loading = new Subject();
        this.loading$ = this.loading.asObservable();
        this.query = new BehaviorSubject(initialQuery);
        this.sort = new BehaviorSubject(initialSort);
        var param$ = combineLatest([this.query, this.sort]);
        this.page$ = param$.pipe(switchMap(function (_a) {
            var _b = __read(_a, 2), query = _b[0], sort = _b[1];
            return _this.pageNumber.pipe(startWith(0), switchMap(function (page) { return _this.endpoint({ page: page, sort: sort, size: _this.pageSize }, query)
                .pipe(indicate(_this.loading)); }));
        }), shareReplay(1));
    }
    PaginationDataSource.prototype.sortBy = function (sort) {
        var lastSort = this.sort.getValue();
        var nextSort = __assign(__assign({}, lastSort), sort);
        this.sort.next(nextSort);
    };
    PaginationDataSource.prototype.queryBy = function (query) {
        var lastQuery = this.query.getValue();
        var nextQuery = __assign(__assign({}, lastQuery), query);
        this.query.next(nextQuery);
    };
    PaginationDataSource.prototype.fetch = function (page) {
        this.pageNumber.next(page);
    };
    PaginationDataSource.prototype.connect = function () {
        return this.page$.pipe(map(function (page) { return page.content; }));
    };
    PaginationDataSource.prototype.disconnect = function () {
    };
    return PaginationDataSource;
}());

/*
 * Public API Surface of ngx-pagination-data-source
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PaginationDataSource };
//# sourceMappingURL=ngx-pagination-data-source.js.map
